# AlignClustConfig.pm
#
# Loads RNAalignClust configuration.
# Authors: Milad Miladi <m.milady@gmail.com>,
#          Alexander Junge <alexander.junge@gmail.com>

package Config::AlignClustConfig;

use strict;
use warnings;

use FindBin;                 # locate this script
use lib "$FindBin::Bin/../lib";  # use the parent directory
use ReadConfigFile;

# For diagnosis
my $print_default_config = 0;
my $print_local_config = 0;
my $print_final_config = 0;


###########################################################

=item load_config()

Loads the default configuration and overwrites all entries set in a local
configuration.

Input:
        $default_config_location - string
        Path to the file storing the local configuration

Output:
        $config_hash_ref - hash reference
        Reference to the hash storing the final configuration.

=cut

###########################################################

sub load_config {
    my $local_config_location = shift;

    my $default_config_location = "$FindBin::Bin/../Config/default.config";
    my $default_config_ref;
    if (-e $default_config_location) {
        $default_config_ref = ReadConfigFile::read_config_file($default_config_location);
    } else {
        die sprintf("Default configuration file %s was not found.", $default_config_location);
    }
    
    if ($print_default_config) {
        ReadConfigFile::print_config($default_config_ref);
        my %default_config = %$default_config_ref;
        my $str_hash_ref = $default_config{'STR_FORMATS'};
        my %str_hash = %$str_hash_ref;
        warn "DOTBRACKET: " . $str_hash{'DOTBRACKET'} . "\n";
        warn "CONSTRAINT: " . $str_hash{'CONSTRAINT'} . "\n\n\n";
    }
    
    my %config_hash = %$default_config_ref;
    
    if (defined $local_config_location) {
        if ( -e $local_config_location ) {
            my $local_config_ref = ReadConfigFile::read_config_file($local_config_location);
            my %local_config = %$local_config_ref;
            if ($print_local_config) {
                ReadConfigFile::print_config($local_config_ref);
                my $str_hash_ref = $local_config{'STR_FORMATS'};
                my %str_hash = %$str_hash_ref;
                warn "DOTBRACKET: " . $str_hash{'DOTBRACKET'} . "\n";
                warn "CONSTRAINT: " . $str_hash{'CONSTRAINT'} . "\n\n\n";
            }
            %config_hash = %{ReadConfigFile::update_config(\%config_hash, $local_config_ref)};
            if ($print_final_config) {
                my $config_hash_ref = \%config_hash;
                ReadConfigFile::print_config($config_hash_ref);
                my $str_hash_ref = $config_hash{'STR_FORMATS'};
                my %str_hash = %$str_hash_ref;
                warn "DOTBRACKET: " . $str_hash{'DOTBRACKET'} . "\n";
                warn "CONSTRAINT: " . $str_hash{'CONSTRAINT'} . "\n\n\n";
            } 
        } else {
            die sprintf("Local configuration file %s was not found.", $local_config_location);
        }
    }
    return \%config_hash;
}

1;
