
# coding: utf-8

# This script is intended to analyze gspan file generated via RNAalignClust pipeline.

# In[10]:

#global & fixed vars
## input file should be in minimal gspan format, here disctinguished with .gspan2
## running the following command would do so, 
## in fact it keeps the first three/four fields for nodes/edges  
##cat file.gspan | awk '{if($1=="v" || $1=="V"){print $1,$2,$3} else if($1=="e"){print $1,$2,$3,$4} else {print $0}}' > file.gspan2
input_data_path = '../../results/20150213-rfam-clique-GraphClust/20150213-GraphClust-rfam-clique.gspan2'
out_analysis_dir = '../../results/20150213-rfam-clique-GraphClust/'
#'./random_split_petfold_struct_stack_seqt.gspan2'

#parameters setting
max_radius=3
min_radius=0
max_distance=5
chunk_size=100
GRAPHCLUST = True

#check file exist
import os.path
if not os.path.isfile(input_data_path):
        raise IOError('file does NOT exist.')
        
def grouper(iterable, n, fillvalue=None):
    "Collect data into fixed-length chunks or blocks"
    # grouper('ABCDEFG', 3, 'x') --> ABC DEF Gxx"
    args = [iter(iterable)] * n
    from itertools import izip_longest
    return izip_longest(*args, fillvalue=fillvalue)

        


# In[2]:

def save_sparse_csr(filename,array):
    np.savez(filename,data = array.data ,indices=array.indices,
             indptr =array.indptr, shape=array.shape )

def load_sparse_csr(filename):
    loader = np.load(filename)
    return csr_matrix((  loader['data'], loader['indices'], loader['indptr']),
                         shape = loader['shape'])


# Extract annotation reference from gspan instance header:

# In[3]:

import re
instances = []
classes = {}
with open(input_data_path, 'r') as f:
    for line in f:
        if line.startswith('t #'): #start of a new instance, line contains tags,etc
            tokens = line.split()
            match_class = ""
            match_subclass = ""
            if (not GRAPHCLUST): ## alignclust gspan header format
                tag = tokens[3] #tag is the column containg instance id
    #             print tag
                matches = re.match(r"random_subalign_all_5_10_(RF.*)\.(\S+)\.folded", tag)
                if (not matches):
                    raise IOError('Cannot find id info from instance header: ' + line )
                match_class = matches.group(1)
                match_subclass = matches.group(2)
            else: ##GraphClust gspan tag
                tag = tokens[8] #tag is the column containg instance id
#                 print tag
                matches = re.match(r"^(RF[0-9]*)_rep\.(\S+)", tag)
                if (not matches):
                    raise IOError('Cannot find id info from instance header: ' + line )
                match_class = matches.group(1)
                match_subclass = matches.group(2)

            instances.append(dict([('class',match_class), ('subclass',match_subclass), ('id',len(instances)+1)]))
            if(not matches.group(1) in classes):
                classes[matches.group(1)]=len(classes)+1
   


# In[4]:

from eden.converter.graph import gspan
graphs = gspan.gspan_to_eden(input = input_data_path, input_type = 'file')


# In[5]:

get_ipython().run_cell_magic(u'time', u'', u"import itertools\nfrom scipy.sparse import vstack\nfrom eden import graph\n\nvector=graph.Vectorizer(r=max_radius,d=max_distance, min_r=min_radius)\n\nX=vector.transform(itertools.islice(graphs, chunk_size), n_jobs=-1)\nprint '1st: Instances: %d Features: %d with an avg of %d features per instance' % (X.shape[0], X.shape[1],  X.getnnz()/X.shape[0])")


# In[ ]:

get_ipython().run_cell_magic(u'time', u'', u'for chunk in grouper(graphs,chunk_size):\n    X_chunk=vector.transform(chunk,n_jobs=-1)\n    X=vstack([X,X_chunk] , format = "csr")\n    print \'Chunk Instances: %d Features: %d with an avg of %d features per instance\' % (X_chunk.shape[0], X_chunk.shape[1],  X_chunk.getnnz()/X_chunk.shape[0])\n    print \'Instances: %d Features: %d with an avg of %d features per instance\' % (X.shape[0], X.shape[1],  X.getnnz()/X.shape[0])\n    if(X.shape[0]>=4500):\n        break\n')


# In[18]:

import numpy as np
save_sparse_csr(out_analysis_dir + "/X_array_" + str(X.shape[0]), X)


# In[11]:

from eden import graph
help(graph.Vectorizer)


# In[ ]:


print X.shape
print 'Instances: %d Features: %d with an avg of %d features per instance' % (X.shape[0], X.shape[1],  X.getnnz()/X.shape[0])
# print classes
y=[ classes[inst['class']] for inst in instances ]


# KNN K Nearest Neighbors clustering of the vectorized/transferred graph according to gspan reference headers

# In[ ]:

get_ipython().run_cell_magic(u'time', u'', u'from sklearn.neighbors import kneighbors_graph\n\nnum_neighbours=4\n\nA = kneighbors_graph(X, num_neighbours)')


# In[ ]:

#draw adjancy graph 
from eden.util import display
display.draw_adjacency_graph(A,y,node_size=80, size=20, layout='graphviz', colormap='rainbow')


# In[8]:

get_ipython().magic(u'pylab qt')
#draw graph
from eden.util import display
import itertools
for g in itertools.islice(graphs, 100,101):
    display.draw_graph(g,size=30,node_border = False, node_size = 200, vertex_color = 'level', colormap='hot')


# In[ ]:



