#!/usr/bin/env python

__author__ = 'Alexander Junge (alexander.junge@gmail.com)'

import argparse
import sys
import re

parser = argparse.ArgumentParser(description=
                                 """Computes the structuredness (ratio of paired bases among all bases) of a consensus
                                 structure present in a file in Stockholm format. The file is read from stdin
                                 and the structuredness is written to stdout.""")
args = parser.parse_args()

consensus_structure_re = re.compile('^#=GC\s+SS_cons\s+(\S+)')
consensus_structure = ""

input_file_handler = sys.stdin
for line in input_file_handler:
    consensus_structure_match = re.match(consensus_structure_re, line)
    if consensus_structure_match:
        consensus_structure += consensus_structure_match.group(1)

base_count = len(consensus_structure)
unpaired_bases = consensus_structure.count('.')
paired_bases = base_count - unpaired_bases

# structuredness is number of paired bases divided by number of bases
# TODO Column with >25% gaps should be removed from alignment before computing this
structuredness = float(paired_bases)/base_count

sys.stdout.write(str(structuredness) + "\n")

