#!/usr/bin/perl
use strict;
use warnings;
use List::Util qw[min max sum];
use Cwd qw(abs_path getcwd);
use FindBin;
use lib "$FindBin::Bin";

# locate perf tool
my $BIN_DIR = abs_path($FindBin::Bin);
my $PERF = "$BIN_DIR/perf";
die "Cannot find perf toll on $PERF" if (!-e $PERF);
open ANNOT, $ARGV[0] or die $!;
my %annotationHash = ();
my $c = 0;
while(<ANNOT>){
	chomp($_);
    $c++;
	$annotationHash{$c} = $_;
}
close ANNOT;

mkdir "/tmp/temp_perf" if(!-d "/tmp/temp_perf");

my $sim_matrix = $ARGV[1];
my $measure_type  = "ROC";
$measure_type = $ARGV[2] if (@ARGV > 2);
chomp $measure_type;
print "Measuring $measure_type for $sim_matrix\n";
my $count = 0;
open SIM, "$sim_matrix" or die $!;
my %rocs = ();
my @all_rocs = ();
open ROCIN, ">perf_plot_roc.in" or die $!;
while(<SIM>){
	my $line = $_;
	chomp ($line);
	my @fields = split('\s', $line);
	($count % 100 == 1) and print ".";
	$count++;
	($count % 1000 == 0) and print $count/1000,"K";
	($count % 5000 == 0) and print "\n";
	my $row_family = $annotationHash{$count};
	my %similarities = ();
	for (my $i = 0; $i <= $#fields; $i++){
		$similarities{$i+1}= $fields[$i];
	}
	open TEMP, ">/tmp/temp_perf/$row_family\_$count" or die $!;
	my $row_string = "";
	foreach my $key (keys %similarities){
		my $family = $annotationHash{$key};
	#	print "$family\t$row_family\n";
		if($row_family eq $family){
			#print TEMP "1 ",$similarities{$key},"\n";
			#print ROCIN "1 ",$similarities{$key},"\n";
			$row_string .= "1 ".$similarities{$key}."\n";
		}
		else{
#			print TEMP "0 ",$similarities{$key},"\n";
#			print ROCIN "0 ",$similarities{$key},"\n";
			$row_string .= "0 ".$similarities{$key}."\n"
		}
	}
	print TEMP $row_string;
	print ROCIN $row_string;
	close TEMP;
	#print "invoke: $PERF -$measure_type < /tmp/temp_perf/$row_family\_$count\n";
	my $roc = `$PERF -$measure_type < /tmp/temp_perf/$row_family\_$count`;
	#system ("perf -plot roc < temp_perf/$row_family\_$count > ");
	#printf "perf out: $roc\n";
	$roc =~ s/$measure_type\s*//;
	push (@{$rocs{$row_family}}, $roc);
	push (@all_rocs, $roc);		
	system "rm /tmp/temp_perf/$row_family\_$count";
}
close SIM;
close ROCIN;
print "\n";
my $avg_roc = sum(@all_rocs)/@all_rocs;
my $non_single_sum = 0;
my $non_single_count = 0;
my $single_count = 0;
my $outfile_name = "$sim_matrix.$measure_type.txt";
open OUTFILE, ">$outfile_name" or die "$!";
#print $#family_rocs+1,"\t",$family,"\t",$family_avg_roc,"\n";
foreach my $family (sort { $a <=> $b } keys %rocs){
	my @family_rocs = @{$rocs{$family}};
	if (@family_rocs >1)
	{
	  $non_single_sum += sum(@family_rocs);
	  $non_single_count += @family_rocs;
	}
	else
	{
		$single_count++;
		#print "#$family has one element\n";
	}
	my $family_avg_roc = sum(@family_rocs)/@family_rocs;
	$family_avg_roc = sprintf("%.3f", $family_avg_roc);
	print OUTFILE $#family_rocs+1,"\t",$family,"\t",$family_avg_roc,"\n";
}
$avg_roc = sprintf("%.3f", $avg_roc);
my $nonsingle_avg_roc = sprintf("%.3f", $non_single_sum/$non_single_count);



print OUTFILE "total $measure_type avg:",$avg_roc,"\n";
print OUTFILE "average over non-singles:" .  $nonsingle_avg_roc . "\n";
print OUTFILE "result:" .  $nonsingle_avg_roc . "\n";
close OUTFILE or die "$!";
print "perf measures were written to $outfile_name\n";

print "total $measure_type avg:",$avg_roc,"\n";
print "$single_count out of " . scalar(@all_rocs) . " families have one element\n";
print "average over non-singles: $nonsingle_avg_roc \n";
print  "result:" .  $nonsingle_avg_roc . "\n";
system "rm -r /tmp/temp_perf";
