# -------------------------------
## calculate identity percentage of two aligned sequences by counting #matches/#alignmnet lenght, column with all gaps is ignored 
import re
## Sequence Identity
#Calculating sequence identity for a pair of sequences
#Note: Normalization, alternatively dive by len short or long sequence 

def sequence_identity(seq1, seq2):
    ## convert all gap chars to "-"
#     print "seq1:{}\nseq2:{}".format(seq1,seq2)
    seq1=re.sub(regx_gaps,'-',str(seq1))
    seq2=re.sub(regx_gaps,'-',str(seq2))
    if (len(seq1) != len (seq2) or len(seq1) == 0):
        raise NameError("Error: assumed aligned sequences with same non zero length")
    total_len = 0
    match_count = 0
    for index in range(len (seq1)):
        if( seq1[index] != "-" or seq2[index] != "-"):
            total_len += 1
            if seq1[index] == seq2[index]:
                match_count += 1
    return 100*match_count/total_len #identity in percent

# -------------------------------
import glob
import os
def remove_files(pattern):
    for fl in glob.glob(pattern):
        os.remove(fl)
        

## -------------
## write subalignmnet of align_dict into an stockholm file, subalignment is selected based on nodes of the given a networkx graph
from Bio.Align import MultipleSeqAlignment
def graph2stockholm(graph, out_file, align_dict):
    if(len(graph) == 0):
        return;
    subalignment_list = []
    output_handle = open(out_file, "w")
    for node_id in graph:
        subalignment_list.append(align_dict[node_id])
    subalignment = MultipleSeqAlignment(subalignment_list)
    AlignIO.write(subalignment, output_handle, "stockholm")
    output_handle.close()
  

#Read alignmnet stockholm file, extract species info from phylo tree file
def alignment2graph():
  ##read stk
  stk_handle = open (stk_name, 'r')
  alignment = AlignIO.read(stk_handle, "stockholm")
  print "Number of sequences: " + str(len(alignment))
  ##capture sequence species from newick tree file
  from ete2 import Tree as eteTree
  tree = eteTree(tree_name);
  # print tree.get_leaves_by_name("0.00_M35060.1/1-72_Saccharomyces_cerevisia..")
  p = re.compile("^\S*_(\S+/\S+\d+)_(\S+)") ## sample input: 0.00_M35060.1/1-72_Saccharomyces_cerevisia..
  id2species = {}
  for leaf in tree.get_leaves():
    seq_id = p.match(leaf.name).group(1)
    seq_spec = p.match(leaf.name).group(2)
    if seq_id in id2species:
	raise NameError ("ERROR: " + seq_id + " ocured more than once, exit.\n")
    else:    
	id2species[seq_id] = seq_spec

  print "Number of species: " + str(len(set (id2species[spec] for spec in id2species)) ) ##totoal number of scpecies
  id2record = {}
  ##check ids against species list
  for record in alignment:
    if not(record.id in id2species):
	raise NameError (record.id + " does exist in tree file")
    id2record[record.id] = record

  ## Create the graph
  from Bio.pairwise2 import format_alignment
  ##create a graph, sequnces as nodes
  import networkx as nx
  G=nx.Graph()

  ## add nodes and edges for pairs of different specieis                   
  for i , seq1 in enumerate(alignment):
    G.add_node(seq1.id)
    for j in range (i+1, len(alignment)):
	seq2 = alignment[j]
	if (id2species[seq1.id] != id2species[seq2.id]):
	    pw_identity = sequence_identity(seq1.seq, seq2.seq)
	    if (pw_identity > LOW_IDENTITY):
		G.add_edge(seq1.id, seq2.id)
		    

  ## merge/delete same species 
  ## we can merge nodes of same specieis with very high similarity, since they are extremly similary removing one of them can also work
  node_list = G.nodes(data=False)
  remove_count = 0
  # print node_list
  for i, node_i in enumerate(node_list):
    for j in range(i+1,len(node_list)):
	node_j = node_list[j]
  #         print node_i, node_j
	if(id2species[node_i] == id2species[node_j]):
	    pw_identity = sequence_identity(id2record[node_i].seq, id2record[node_j].seq)
	    if (pw_identity > EXTREME_IDENTITY):
		G.remove_node(node_i)
		print "{} removed, identity:{}".format(node_i, pw_identity) if (i_debug)
		remove_count+= 1
		break

  return G;

#-----------------------
def graph2clique-alignment(graph)
  import random
  clique_id = 0
  remove_files("./" + fam_id + ".[0-9]*") 
  while(graph.number_of_nodes >= MIN_SUBALIGN_SIZE):
    clique_list = list(nx.find_cliques(graph))
    max_clique = random.choice(clique_list) ## randomly select a max_clique
    if len(max_clique) < MIN_SUBALIGN_SIZE:
	print "Max clique of remainig graph had too few elements"
	break
    graph2stockholm (max_clique, graph, fam_id + "." + str(clique_id), id2record)
    clique_id += 1
    graph.remove_nodes_from(max_clique)
    print "clique[{}]: {} nodes extracted".format(clique_id, len(clique_list))

  print "{} cliques and subalignmnets generated ".format(clique_id)

# flatten 2d list
# all_clique_nodes = [node for clique in clique_list for node in clique]
# print len(set(all_clique_nodes))
