#!/usr/bin/env bash
# Run this script inside Docker container when run Docker container with the following command:
# docker run -it -v "$CURRENT_WORKING_DIRECTORY"/cluster_evaluation:/cluster_evaluation -h dockersgeserver mmiladi/rnascclust:v"$IMAGE_VERSION"
# Inside container, the present script can be executed by:
# bash rnascclust/bin/clustering/run_clustering_docker.sh
perform_knn_analysis=false
docker_out="cluster_evaluation"
output_subdir_GC="RESULTS_GC"
data_set_names=( "cliques-low"  "cliques-medium" "cliques-high" "rfamome" )

for ((i=0; i < ${#data_set_names[@]}; i++))
do
    data_set=${data_set_names[$i]}
    out_dir_RSC="$docker_out/$data_set"
    out_dir_GC="$docker_out/$data_set/$output_subdir_GC"

    echo "Start RNAscClust for $data_set"
    mkdir -p "$out_dir_RSC"/RESULTS_SC
    RSC_out="$out_dir_RSC"/RESULTS_SC/log
    RSC_err="$out_dir_RSC"/RESULTS_SC/err
    /rnascclust/bin/run_rnascclust.py --alignments-dir "/rnascclust/data/$data_set/alignments/" --rep-fasta "/rnascclust/data/$data_set/representatives/representatives.fa" --predict-structure --result-path "/$out_dir_RSC/" --local-config /rnascclust/Config/docker-sge.config --sge 1>"$RSC_out" 2>"$RSC_err"
    echo "Start clustering evaluation"
    RSC_eval_dir="$out_dir_RSC"/RESULTS_SC/cluster_evaluation
    RSC_eval_log="$RSC_eval_dir"/log
    RSC_eval_err="$RSC_eval_dir"/err
    mkdir -p "$RSC_eval_dir"

    if [ "$perform_knn_analysis" = false ]; then
    	echo "Knn Analysis does run by default, since scikit-learn requires high amount of physical memory"
        python2 /rnascclust/bin/clustering/cluster_evaluation.py --no_knn_analysis "$out_dir_RSC"/RESULTS_SC "$RSC_eval_dir" 1>"$RSC_eval_log" 2>"$RSC_eval_log"
    else
        python2 /rnascclust/bin/clustering/cluster_evaluation.py "$out_dir_RSC"/RESULTS_SC "$RSC_eval_dir" 1>"$RSC_eval_log" 2>"$RSC_eval_log"
    fi

    echo "Start GraphClust for $data_set"
    mkdir -p "$out_dir_GC"
    GC_out="$out_dir_GC"/log
    GC_err="$out_dir_GC"/err
    /usr/local/user/GraphClust-0.7.6/bin/MASTER_GraphClust.pl --fasta "/rnascclust/data/$data_set/representatives/representatives.fa" --root-dir "/$out_dir_GC/" --config /rnascclust/bin/GraphClust/config.cnf -sge -no-ssh 1>"$GC_out" 2>"$GC_err"
    echo "Start clustering evaluation"
    GC_eval_dir="$out_dir_GC"/cluster_evaluation
    GC_eval_log="$GC_eval_dir"/log
    GC_eval_err="$GC_eval_dir"/err
    mkdir -p "$GC_eval_dir"

    if [ "$perform_knn_analysis" = false ]; then
        python2 /rnascclust/bin/clustering/cluster_evaluation.py --graph_clust --no_knn_analysis "$out_dir_GC" "$GC_eval_dir" 1>"$GC_eval_log" 2>"$GC_eval_log"
    else
        python2 /rnascclust/bin/clustering/cluster_evaluation.py --graph_clust "$out_dir_GC" "$GC_eval_dir" 1>"$GC_eval_log" 2>"$GC_eval_log"
    fi
done

echo "Producing Result figures and tables in paper."
if [ "$perform_knn_analysis" = false ]; then
    python2 /rnascclust/bin/clustering/cluster_evaluation_manuscript.py --no_knn_analysis
else
    python2 /rnascclust/bin/clustering/cluster_evaluation_manuscript.py
fi